@@ -54,13 +54,9 @@ func TestOKWithScaleUp(t *testing.T) {
 	clusterstate := NewClusterStateRegistry(provider, ClusterStateRegistryConfig{
 		MaxTotalUnreadyPercentage: 10,
 		OkTotalUnreadyCount:       1,
+		MaxNodeProvisionTime:      time.Minute,
 	}, fakeLogRecorder, newBackoff())
-	clusterstate.RegisterScaleUp(&ScaleUpRequest{
-		NodeGroup:       provider.GetNodeGroup("ng1"),
-		Increase:        4,
-		Time:            now,
-		ExpectedAddTime: now.Add(time.Minute),
-	})
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), 4, time.Now())
 	err := clusterstate.UpdateNodes([]*apiv1.Node{ng1_1, ng2_1}, nil, now)
 	assert.NoError(t, err)
 	assert.True(t, clusterstate.IsClusterHealthy())
@@ -99,6 +95,7 @@ func TestEmptyOK(t *testing.T) {
 	clusterstate := NewClusterStateRegistry(provider, ClusterStateRegistryConfig{
 		MaxTotalUnreadyPercentage: 10,
 		OkTotalUnreadyCount:       1,
+		MaxNodeProvisionTime:      time.Minute,
 	}, fakeLogRecorder, newBackoff())
 	err := clusterstate.UpdateNodes([]*apiv1.Node{}, nil, now.Add(-5*time.Second))
 	assert.NoError(t, err)
@@ -107,12 +104,9 @@ func TestEmptyOK(t *testing.T) {
 	assert.False(t, clusterstate.IsNodeGroupScalingUp("ng1"))
 
 	provider.AddNodeGroup("ng1", 0, 10, 3)
-	clusterstate.RegisterScaleUp(&ScaleUpRequest{
-		NodeGroup:       provider.GetNodeGroup("ng1"),
-		Increase:        3,
-		Time:            now.Add(-3 * time.Second),
-		ExpectedAddTime: now.Add(1 * time.Minute),
-	})
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), 3, now.Add(-3*time.Second))
+	//	clusterstate.scaleUpRequests["ng1"].Time = now.Add(-3 * time.Second)
+	//	clusterstate.scaleUpRequests["ng1"].ExpectedAddTime = now.Add(1 * time.Minute)
 	err = clusterstate.UpdateNodes([]*apiv1.Node{}, nil, now)
 
 	assert.NoError(t, err)
@@ -334,12 +328,7 @@ func TestExpiredScaleUp(t *testing.T) {
 		OkTotalUnreadyCount:       1,
 		MaxNodeProvisionTime:      2 * time.Minute,
 	}, fakeLogRecorder, newBackoff())
-	clusterstate.RegisterScaleUp(&ScaleUpRequest{
-		NodeGroup:       provider.GetNodeGroup("ng1"),
-		Increase:        4,
-		Time:            now.Add(-3 * time.Minute),
-		ExpectedAddTime: now.Add(-1 * time.Minute),
-	})
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), 4, now.Add(-3*time.Minute))
 	err := clusterstate.UpdateNodes([]*apiv1.Node{ng1_1}, nil, now)
 	assert.NoError(t, err)
 	assert.True(t, clusterstate.IsClusterHealthy())
@@ -620,15 +609,11 @@ func TestScaleUpBackoff(t *testing.T) {
 	clusterstate := NewClusterStateRegistry(provider, ClusterStateRegistryConfig{
 		MaxTotalUnreadyPercentage: 10,
 		OkTotalUnreadyCount:       1,
+		MaxNodeProvisionTime:      120 * time.Second,
 	}, fakeLogRecorder, newBackoff())
 
 	// After failed scale-up, node group should be still healthy, but should backoff from scale-ups
-	clusterstate.RegisterScaleUp(&ScaleUpRequest{
-		NodeGroup:       provider.GetNodeGroup("ng1"),
-		Increase:        1,
-		Time:            now.Add(-3 * time.Minute),
-		ExpectedAddTime: now.Add(-1 * time.Minute),
-	})
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), 1, now.Add(-180*time.Second))
 	err := clusterstate.UpdateNodes([]*apiv1.Node{ng1_1, ng1_2, ng1_3}, nil, now)
 	assert.NoError(t, err)
 	assert.True(t, clusterstate.IsClusterHealthy())
@@ -642,12 +627,8 @@ func TestScaleUpBackoff(t *testing.T) {
 	assert.True(t, clusterstate.IsNodeGroupSafeToScaleUp(ng1, now))
 
 	// Another failed scale up should cause longer backoff
-	clusterstate.RegisterScaleUp(&ScaleUpRequest{
-		NodeGroup:       provider.GetNodeGroup("ng1"),
-		Increase:        1,
-		Time:            now.Add(-2 * time.Minute),
-		ExpectedAddTime: now.Add(-1 * time.Second),
-	})
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), 1, now.Add(-121*time.Second))
+
 	err = clusterstate.UpdateNodes([]*apiv1.Node{ng1_1, ng1_2, ng1_3}, nil, now)
 	assert.NoError(t, err)
 	assert.True(t, clusterstate.IsClusterHealthy())
@@ -658,12 +639,7 @@ func TestScaleUpBackoff(t *testing.T) {
 	assert.False(t, clusterstate.IsNodeGroupSafeToScaleUp(ng1, now))
 
 	// The backoff should be cleared after a successful scale-up
-	clusterstate.RegisterScaleUp(&ScaleUpRequest{
-		NodeGroup:       provider.GetNodeGroup("ng1"),
-		Increase:        1,
-		Time:            now,
-		ExpectedAddTime: now.Add(time.Second),
-	})
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), 1, now)
 	ng1_4 := BuildTestNode("ng1-4", 1000, 1000)
 	SetNodeReadyState(ng1_4, true, now.Add(-1*time.Minute))
 	provider.AddNode("ng1", ng1_4)
@@ -672,7 +648,7 @@ func TestScaleUpBackoff(t *testing.T) {
 	assert.True(t, clusterstate.IsClusterHealthy())
 	assert.True(t, clusterstate.IsNodeGroupHealthy("ng1"))
 	assert.True(t, clusterstate.IsNodeGroupSafeToScaleUp(ng1, now))
-	assert.False(t, clusterstate.nodeGroupBackoffInfo.IsBackedOff(ng1, now))
+	assert.False(t, clusterstate.backoff.IsBackedOff(ng1, nil, now))
 }
 
 func TestGetClusterSize(t *testing.T) {
@@ -726,6 +702,50 @@ func TestGetClusterSize(t *testing.T) {
 	assert.Equal(t, 30, targetSize)
 }
 
+func TestUpdateScaleUp(t *testing.T) {
+	now := time.Now()
+	later := now.Add(time.Minute)
+
+	provider := testprovider.NewTestCloudProvider(nil, nil)
+	provider.AddNodeGroup("ng1", 1, 10, 5)
+	fakeClient := &fake.Clientset{}
+	fakeLogRecorder, _ := utils.NewStatusMapRecorder(fakeClient, "kube-system", kube_record.NewFakeRecorder(5), false)
+	clusterstate := NewClusterStateRegistry(
+		provider,
+		ClusterStateRegistryConfig{
+			MaxTotalUnreadyPercentage: 10,
+			OkTotalUnreadyCount:       1,
+			MaxNodeProvisionTime:      10 * time.Second,
+		},
+		fakeLogRecorder,
+		newBackoff())
+
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), 100, now)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].Increase, 100)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].Time, now)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].ExpectedAddTime, now.Add(10*time.Second))
+
+	// expect no change of times on negative delta
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), -20, later)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].Increase, 80)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].Time, now)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].ExpectedAddTime, now.Add(10*time.Second))
+
+	// update times on positive delta
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), 30, later)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].Increase, 110)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].Time, later)
+	assert.Equal(t, clusterstate.scaleUpRequests["ng1"].ExpectedAddTime, later.Add(10*time.Second))
+
+	// if we get below 0 scalup is deleted
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), -200, now)
+	assert.Nil(t, clusterstate.scaleUpRequests["ng1"])
+
+	// If new scalup is registered with negative delta nothing should happen
+	clusterstate.RegisterOrUpdateScaleUp(provider.GetNodeGroup("ng1"), -200, now)
+	assert.Nil(t, clusterstate.scaleUpRequests["ng1"])
+}
+
 func TestIsNodeStillStarting(t *testing.T) {
 	testCases := []struct {
 		desc           string
